<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Comment controller class file.
 *
 * @author Vitalii Kovalchuk <vitaly.kovalchuk@oberig.com>
 */

/**
 * Comment controller class
 *
 * @author Vitalii Kovalchuk <vitaly.kovalchuk@oberig.com>
 *
 */
class BulkController extends BaseController
{
    /**
     *
     * @var _users
     * @desc user AR container
     */
    private $_users;

    /**
     * Defines filters
     *
     * @return array
     */
    public function filters()
    {
        return array(
            'accessControl'
        );
    }

    public function accessRules()
    {
        return array(
            array(
                'allow',
                'roles' => array('Admin')
            ),
            array(
                'deny',
                'users' => array('*')
            )
        );
    }

    /**
     * Inits data
     *
     * @return array
     */
    public function init()
    {
        $logRouter = Yii::app()->getComponent('log');
        Yii::getLogger()->detachEventHandler('onFlush', array($logRouter, 'collectLogs'));
        Yii::app()->detachEventHandler('onEndRequest', array($logRouter, 'processLogs'));
    }


    /**
     * default action
     * will show module menu
     *
     * @return bool
     */
    public function actionIndex()
    {
        $model = new GenerationForm();

        $usersQty = User::model()->count('is_bulk = :is_bulk', array(':is_bulk' => true));

        $likesQty = Yii::app()->db->createCommand()
            ->select('count(*)')
            ->from(Like::model()->tableName())
            ->join(User::model()->tableName() . ' as u', 'u.id = user_id')
            ->where('is_bulk = :is_bulk', array(':is_bulk' => true))
            ->queryScalar();

        $commentsQty = Yii::app()->db->createCommand()
            ->select('count(*)')
            ->from(Comment::model()->tableName())
            ->join(User::model()->tableName() . ' as u', 'u.id = user_id')
            ->where('is_bulk = :is_bulk', array(':is_bulk' => true))
            ->queryScalar();

        if (isset($_POST['GenerationForm'])) {
            $model->attributes = $_POST['GenerationForm'];
            if ($model->validate()) {
                $this->redirect('/bulk/bulk/generate?' . http_build_query($_POST));
            }
        }

        $this->render('index', array('model' => $model, 'usersQty' => $usersQty, 'likesQty' => $likesQty, 'commentsQty' => $commentsQty));
    }

    /**
     * action for displaying generation form and some statistics
     *
     * @return void
     */
    public function actionGenerate()
    {
        if (isset($_GET['GenerationForm'])) {
            $model = new GenerationForm();
            $model->attributes = $_GET['GenerationForm'];
            set_time_limit(0);
            $start = microtime(true);
            $usersCreated = 0;
            if (!empty($model->usersQty)) {
                $usersCreated = $this->_generateUsers($model->usersQty);
            }

            $likesCreated = 0;
            if (!empty($model->likesQty)) {
                if (($likesCreated = $this->_generateLikes($model->likesQty, $model->likesContentId)) === false) {
                    Yii::app()->user->setFlash('error', BulkModule::t('There is no bulk users(free bulk users)! Cant create likes'));
                }
            }

            $commentsCreated = 0;
            if (!empty($model->commentsQty)) {
                if (($commentsCreated = $this->_generateComments($model->commentsQty, $model->commentsContentId, $model->commentsForEach)) === false) {
                    Yii::app()->user->setFlash('error', BulkModule::t('There is no bulk users! Cant create comments'));
                }
            }

            $totalTime = round(microtime(true) - $start, 4);
            $this->render(
                'bulk.views.bulk.generate',
                array('usersCreated' => $usersCreated, 'likesCreated' => $likesCreated, 'commentsCreated' => $commentsCreated, 'time' => $totalTime)
            );
        } else {
            $this->redirect('index');
        }
    }

    /**
     * action for users generation
     *
     * @param int $qty required user's quantity
     *
     * @return int
     */
    private function _generateUsers($qty)
    {
        $stamp = time();
        $db = Yii::app()->db->beginTransaction();
        $profile = new Profile;
        $faker = new Faker();
        $locale = include Yii::getPathOfAlias('system') . '/i18n/data/en.php';
        $countries = array_values($locale['territories']);

        $createdQty = 0;
        for ($i = 1; $i <= $qty; $i++) {
            $user = new User();
            $user->is_bulk = true;
            $user->activkey = md5('user' . $i . '_' . $stamp);
            $user->attributes = array(
                'username' => 'user' . $i . '_' . $stamp,
                'password' => md5('user' . $i . '_' . $stamp),
                'email' => 'user' . $i . '_' . $stamp . '@example.com',
                'status' => 1,
            );
            if ($user->save()) {
                $firstname = $faker->Name->prefix() . $faker->Name->first_name();
                $lastname = $faker->Name->surname() . ' ' . $faker->Name->suffix();

                $profile->setIsNewRecord(true);
                $profile->regMode = true;
                $profile->user_id = $user->id;
                foreach (Profile::model()->getFields() as $field) {
                    $value = null;
                    if ($field->range) {
                        $pos = strpos($field->range, '-');
                        if ($pos !== false) {
                            $tag = substr($field->range, 0, $pos);
                            switch ($tag) {
                                case 'checkbox':
                                    $field->range = substr($field->range, $pos + 1);
                                    break;
                                case 'radio':
                                    $field->range = substr($field->range, $pos + 1);
                                    break;
                                case 'select':
                                    $field->range = substr($field->range, $pos + 1);
                                    break;
                                default:
                                    break;
                            }
                        }
                        $value = array_rand(Profile::range($field->range));
                    } elseif ($field->field_type == 'VARCHAR') {
                        $value = $this->_randStr(rand($field->field_size_min, $field->field_size));
                    } elseif ($field->field_type == 'DATE') {
                        $value = date('Y-m-d', $stamp - 20 * 368 * 24 * 60 * 60);
                    } else {
                        $value = rand($field->field_size_min, $field->field_size);
                    }
                    $profile->{$field->varname} = $value;
                }

                $profile->attributes = array(
                    'firstname' => $firstname,
                    'lastname' => $lastname,
                );
                if ($profile->save()) {
                    $createdQty++;
                } else {
                    //var_dump($profile->getErrors());
                }

            } else {
                //var_dump($user->getErrors());
            }
        }
        $db->commit();

        return $createdQty;
    }

    /**
     * Action for Likes generation
     *
     * @param int        $qty       required likes quantity
     * @param int|string $contentId string contentId that should be
     *
     * @return bool
     */
    private function _generateLikes($qty, $contentId = 0)
    {
        $userIds = false;
        $bulkUsers = Yii::app()->db->createCommand()
            ->select('u.id')
            ->from(User::model()->tableName() . ' u');
        if ($contentId) {
            $bulkUsers->where(
                'is_bulk = :is_bulk and id not in ( SELECT user_id from {{likes}} where content_id=:content_id)',
                array(':content_id' => $contentId, ':is_bulk' => true)
            );
        } else {
            $bulkUsers->where('is_bulk = :is_bulk', array(':is_bulk' => true));
        }
        $bulkUsers = $bulkUsers->queryAll();
        if (!empty($bulkUsers)) {
            $userIds = array_map(
                function ($user) {
                    return $user['id'];
                }, $bulkUsers
            );
        }

        if ($userIds) {
            $transaction = Yii::app()->db->beginTransaction();
            $createdQty = 0;
            $like = new Like();
            $userCount = count($userIds);
            for ($i = 0; $i < $qty; $i++) {
                if (!empty($contentId) && $i >= $userCount) {
                    break;
                }
                $like->id = null;
                $like->setIsNewRecord(true);
                $like->user_id = $userIds[$i % $userCount];
                $like->content_id = empty($contentId) ? uniqid('', true) : $contentId;
                $like->createdate = date('Y-m-d H:i:s');
                if ($like->save()) {
                    $createdQty++;
                }
            }
            $transaction->commit();
            return $createdQty;
        } else {
            return false;
        }
    }

    /**
     * Action for Comments generation
     *
     * @param int        $qty         required likes quantity
     * @param int|string $contentId   string contentId that should be
     * @param bool       $forEachUser defines if given quantity is for each user or total
     *
     * @return bool
     */
    private function _generateComments($qty, $contentId = 0, $forEachUser = false)
    {
        if ($users = $this->_getBulkUsers()) {
            $createdQty = 0;
            $userCount = count($users);
            $faker = new Faker();
            if ($forEachUser) {
                foreach ($users as $u) {
                    for ($i = 0; $i < $qty; $i++) {
                        $comment = new Comment();
                        $comment->setIsNewRecord(true);
                        $comment->content_id = empty($contentId) ? uniqid('', true) : $contentId;
                        $comment->user_id = $u['id'];
                        $comment->email = $u['email'];
                        $comment->subject = $faker->Lorem->sentence(1);
                        $comment->description = $faker->Lorem->sentence(20);
                        $comment->language = 'en';
                        $comment->status = Comment::APPROVED;
                        $comment->notify_moderator = true;
                        $comment->notify_user = true;
                        if ($comment->save()) {
                            $createdQty++;
                        }
                    }
                }
            } else {
                for ($i = 0; $i < $qty; $i++) {
                    $comment = new Comment();
                    $comment->setIsNewRecord(true);
                    $comment->content_id = empty($contentId) ? uniqid('', true) : $contentId;
                    $comment->user_id = $users[$i % $userCount]['id'];
                    $comment->email = $users[$i % $userCount]['email'];
                    $comment->subject = $faker->Lorem->sentence(1);
                    $comment->description = $faker->Lorem->sentence(20);
                    $comment->language = 'en';
                    $comment->status = Comment::APPROVED;
                    $comment->notify_moderator = true;
                    $comment->notify_user = true;
                    if ($comment->save()) {
                        $createdQty++;
                    }
                }
            }
            return $createdQty;
        } else {
            return false;
        }
    }

    /**
     * private method that gets list if user ids generated by this module
     *
     * @return bool | array
     */
    private function _getBulkUsersIds()
    {
        $bulkUsers = Yii::app()->db->createCommand()
            ->select('id')
            ->from(User::model()->tableName())
            ->where('is_bulk = :is_bulk', array(':is_bulk' => true))
            ->queryAll(false);
        if (!empty($bulkUsers)) {
            $userIds = array_map(
                function ($user) {
                    return $user[0];
                }, $bulkUsers
            );
            return $userIds;
        }
        return false;
    }

    /**
     * private method that gets list if users generated by this module
     *
     * @return bool | array
     */
    private function _getBulkUsers()
    {
        $bulkUsers = Yii::app()->db->createCommand()
            ->select('id, email')
            ->from(User::model()->tableName())
            ->where('is_bulk = :is_bulk', array(':is_bulk' => true))
            ->limit(10000)
            ->queryAll(false);
        if (!empty($bulkUsers)) {

            $usersData = array();
            foreach ($bulkUsers as $user) {
                $usersData[] = array('id' => $user[0], 'email' => $user[1]);
            }

            return $usersData;
        }
        return false;
    }

    /**
     * private method that gets list if users generated by this module
     *
     * @param string $type defines what type of data should be deleted
     *
     * @return bool | array
     */
    public function actionClear($type)
    {
        $usersQty = 0;
        $likesQty = 0;
        $commentsQty = 0;

        if ($userIds = $this->_getBulkUsersIds()) {
            $userIdsStr = implode(',', $userIds);
            if ($type == 'all') {
                $likesQty = Yii::app()->db->createCommand()->delete(Like::model()->tableName(), "user_id in ($userIdsStr)");
                $commentsQty = Yii::app()->db->createCommand()->delete(Comment::model()->tableName(), "user_id in ($userIdsStr)");
                Yii::app()->db->createCommand()->delete(Profile::model()->tableName(), "user_id in ($userIdsStr)");
                $usersQty = Yii::app()->db->createCommand()->delete(User::model()->tableName(), "is_bulk = :is_bulk", array(':is_bulk' => true));
            } elseif ($type == 'likes') {
                $likesQty = Yii::app()->db->createCommand()->delete(Like::model()->tableName(), "user_id in ($userIdsStr)");
            } elseif ($type == 'comments') {
                $commentsQty = Yii::app()->db->createCommand()->delete(Comment::model()->tableName(), "user_id in ($userIdsStr)");
            }
        }

        $this->render('bulk.views.bulk.clear', array('usersQty' => $usersQty, 'likesQty' => $likesQty, 'commentsQty' => $commentsQty));

    }

    /**
     * function will return string of chars with given length
     *
     * @param int $length length of string to return
     *
     * @return string
     */
    private function _randStr($length = 0)
    {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, strlen($characters) - 1)];
        }
        return $randomString;
    }
}
