<?php
class FGValidator
{
	/*public $validationTypes = array(
		'number',
		'telephone',
		'socialsecurity',
		'date',
		'time',
		'email'
	);*/
	
	public function validate($validationType, $model, $name, $value)
	{
		$method = 'validate'.ucfirst($validationType);
		if(method_exists($this, $method)) {
			$this->$method($model, $name, $value);
		}
	}
	
	public function validateElement($elementType, $model, $name, $value)
	{
		$method = 'validateElement'.ucfirst($elementType);
		if(method_exists($this, $method)) {
			$this->$method($model, $name, $value);
		}
	}
	
	public function addError($object,$attribute,$message,$params=array())
	{
		$element = $object->getElementFromJSONByName($attribute);
		$params['{attribute}'] = isset($element['label']) && $element['label'] !== ''? $element['label'] : $attribute;
		$object->addError($attribute, strtr($message,$params));
	}
	
	/* Element validators */
	public function validateElementCaptcha($model, $name, $value)
	{
		$config = FgModule::getConfig();
		#include dirname(__FILE__).'/../assets/elements/captcha/securimage/securimage.php';
		#$securimage = new Securimage();
		#if($securimage->check($value) == false) {
		#	$message = FgModule::t('captcha-invalid', 'validation');
		#	$this->addError($model,$name,$message);
		#}
		$code = Yii::app()->controller->createAction('captcha')->verifyCode;
		if($value != $code) {
			$message = FgModule::t('captcha-invalid', 'validation');
			$this->addError($model,$name,$message);
		}
	}
	
	/* Validators */
	
	public function validateNumber($model, $name, $value)
	{
		$validator = new CNumberValidator();
		if(!preg_match($validator->numberPattern, strval($value)))
		{
			$message = FgModule::t('number-invalid', 'validation');
			$this->addError($model,$name,$message);
		}
	}
	
	public function validateTelephone($model, $name, $value)
	{
		$valid = true;
		if(preg_match('/^[\s\d\+\/]*$/', $value) == false) {
			$valid = false;
		} else {
			if(preg_match('/\d+/', $value) == false)
				$valid = false;
		}
		if($valid == false) {
			$message = FgModule::t('telephone-invalid', 'validation');
			$this->addError($model,$name,$message);
		}
	}
	
	public function validateSocialsecurity($model, $name, $value)
	{
		$valid = true;
		if(preg_match('/^\s*\d{10}\s*$/', $value) == false) {
			$valid = false;
		} else {
			preg_match('/(\d{6})\s*$/', $value, $m);
			$timestamp=CDateTimeParser::parse($m[1],'ddMMyy',array('month'=>1,'day'=>1,'hour'=>0,'minute'=>0,'second'=>0));
			if($timestamp === false || $timestamp >= time()) {
				$valid = false;
			}
		}
		if($valid == false) {
			$message = FgModule::t('socialsecurity-invalid', 'validation');
			$this->addError($model,$name,$message);
		}
	}
	
	public function validateDate($model, $name, $value)
	{
		$validator = new CDateValidator();
		$timestamp=CDateTimeParser::parse($value,'dd.MM.yyyy',array('month'=>1,'day'=>1,'hour'=>0,'minute'=>0,'second'=>0));
		if($timestamp === false) {
			$message = FgModule::t('date-invalid', 'validation');
			$this->addError($model,$name,$message);
		}
	}
	
	public function validateTime($model, $name, $value)
	{
		$valid = true;
		if(preg_match('/^\s*\d{2}:\d{2}\s*$/', $value) == false) {
			$valid = false;
		} else {
			$time = explode(':', $value);
			if(CTimestamp::isValidTime($time[0], $time[1], 0) == false)
				$valid = false;
		}
		if($valid == false) {
			$message = FgModule::t('time-invalid', 'validation');
			$this->addError($model,$name,$message);
		}
	}
	
	public function validateEmail($model, $name, $value)
	{
		$validator = new CEmailValidator();
		if($validator->validateValue($value) == false) {
			$message = FgModule::t('email-invalid', 'validation');
			$this->addError($model,$name,$message);
		}
	}
}
