<?php

/**
 * An Implementation of CBaseUrlRule for the URL manager to check all requests
 * if they match a nice URL in the repository.
 */
class NiceUrlRule extends CBaseUrlRule
{
    public $controllerAction = 'contentSource/renderer/dynamic';

    /**
     * Parses a URL based on this rule.
     * @param CUrlManager $manager the URL manager
     * @param CHttpRequest $request the request object
     * @param string $pathInfo path info part of the URL (URL suffix is already removed based on {@link CUrlManager::urlSuffix})
     * @param string $rawPathInfo path info that contains the potential URL suffix
     * @return mixed the route that consists of the controller ID and action ID. False if this rule does not apply.
     */
    public function parseUrl($manager, $request, $pathInfo, $rawPathInfo)
    {
        if ($pathInfo === '') {
            return false;
        }

        $contentSource = Yii::app()->getModule('contentSource')->getContentSource();
        $niceUrlModule = Yii::app()->getModule('niceurl');

        if (!($contentSource instanceof DynamicContentSource) || $niceUrlModule == null) {
            return false;
        }

        $filename = basename($pathInfo);

        // Nice URLs must not have a dot in their filename
        if (!$niceUrlModule->allowDots && strpos($filename, '.') !== false) {
            return false;
        }

        // Do a Repository look up if the file is not cached
        //if (!file_exists($contentSource->fileAccessible('/' . $pathInfo, $$contentSource->cacheFolder))) {
        if (!$contentSource->isCachedObjectStillValid('/' . $pathInfo)) {
            $cmsObject = Yii::app()->repositoryApi->getCmsObjectCanBeExternal('/' . $pathInfo);

            if (!$cmsObject || $cmsObject['obj_type'] != "10007") {
                // Doesn't exist or is not a page
                return false;
            }
        } else {            
            $patterns = $niceUrlModule->mediaPatterns;
            // Check its type and wrap it if necessary
            if (!is_array($patterns)) {
                $patterns = (is_string($patterns)) ? array($patterns) : array();
            }

            // Check if it is an media-element that shouldn't be output
            foreach ($patterns as $pattern) {
                if (preg_match($pattern, $pathInfo)) {
                    return false;
                }
            }
        }

        // CR object found for nice URL
        $_GET['path'] = $pathInfo;

        return $this->controllerAction;
    }

    /**
     * Creates a URL based on this rule.
     * @param CUrlManager $manager the manager
     * @param string $route the route
     * @param array $params list of parameters (name=>value) associated with the route
     * @param string $ampersand the token separating name-value pairs in the URL.
     * @return mixed the constructed URL. False if this rule does not apply.
     */
    public function createUrl($manager, $route, $params, $ampersand)
    {
        return false;
    }
}
